<?php
/*
 * Plugin Name: Advanced Order Export  - Custom CSV Format, fixed width text
 * Description: Custom Export for CSV Format, requires Advanced Order Export For WooCommerce
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Woe_Custom_CSV_Format {

    function __construct() {

        //hooks for UI
        add_action("woe_settings_above_buttons", array($this, "draw_options") );

        //before start export
        add_filter('woe_settings_validate_defaults', function ($settings) {
            if( !empty($settings['custom_txt_format']) ) {
                add_filter("woe_csv_custom_output_func", array($this, 'format'), 10, 7 );
            }
            return $settings;
        });
    }

    function draw_options($settings) {
        $selected = !empty($settings[ 'custom_txt_format' ]) ? 'checked': '';
        echo '<br><br>
        <input type=hidden name="settings[custom_txt_format]" value="0">
        <input type=checkbox name="settings[custom_txt_format]" value="1" '. $selected .'>
        Custom CSV format <br><br>';
    }

    public function format($custom_output, $handle, $data, $delimiter, $linebreak, $enclosure, $is_header) {

        if ($is_header) {
            return true;
        }

        $order = new WC_Order(WC_Order_Export_Data_Extractor::$current_order);

        fwrite($handle, '8730-PONOOUT'."\n");
        fwrite($handle, implode('', $this->get_header_row_by_order($order))."\n");

        $i = 0;

        foreach ($order->get_items('line_item') as $item) {
            fwrite($handle, implode('', $this->get_body_row_by_product_line_item($order, $i, $item, $order->get_product_from_item($item)))."\n");
            $i++;
        }

        return true;
    }

    protected function get_header_row_by_order($order) {

        return array(
            $this->format_str('HEA', 3),
            $this->format_str($order->get_order_number(), 8),
            $this->format_str($order->get_user_id(), 7),
            $this->format_str($order->get_shipping_first_name(), 30),
            $this->format_str($order->get_shipping_last_name(), 30),
            $this->format_str($order->get_shipping_address_1(), 30),
            $this->format_str($order->get_shipping_address_2(), 30),
            $this->format_str('', 30),
            $this->format_str('', 12),
            $this->format_str($order->get_shipping_city(), 30),
            $this->format_str($order->get_shipping_postcode(), 20),
            $this->format_str($order->get_shipping_country(), 2),
            $this->format_str($order->get_user_id(), 30),
            $this->format_str($order->get_billing_phone(), 20),
            $this->format_str('', 20),
            $this->format_str('', 20),
            $this->format_str($order->get_currency(), 3),
            (new \DateTime($order->get_date_created(), new \DateTimeZone('UTC')))->format('Y-n-j'),
            $this->format_str('', 3),
            $this->format_str('', 2),
            $this->format_str('', 5),
            $this->format_str(number_format((float)$order->get_total(), 2, '.', ''), 22),
        );
    }

    protected function get_body_row_by_product_line_item($order, $key, $item, $product) {

        return array(
            $this->format_str('ROW', 3),
            $this->format_str($key, 6),
            $this->format_str('', 2),
            $this->format_str($product->get_sku(), 20),
            $this->format_str($product->get_name(), 30),
            $this->format_str('', 30),
            $this->format_str($product->get_sku(), 20),
            $this->format_str('', 3),
            (new \DateTime($order->get_date_created(), new \DateTimeZone('UTC')))->format('Ynj'),
            $this->format_str(number_format((float)$item->get_total(), 2, '.', ''), 22),
            $this->format_str(number_format((int)$item->get_quantity(), 3, '.', ''), 11),
            $this->format_str('', 3),
            $this->format_str('', 30),
            $this->format_str('', 2),
            $this->format_str('', 30),
            $this->format_str(number_format((float)$product->get_weight(), 3, '.', ''), 8),
            $this->format_str('2', 1),
            $this->format_str('', 10),
        );
    }

    protected function format_str($str, $length) {
        return $str.str_repeat(' ', (int)$length - mb_strlen($str));
    }

}

new Woe_Custom_CSV_Format();